	org $FA0000
//****************************************************************************
	include "defines.s"

//****************************************************************************
	// Bit 0 1 (24)-If set, the initialization code is performed before the initialization of the interrupt vectors and display memory.
	// Bit 1 2 (25)-If set, cartridge initialization is performed before GEMDOS is initialized.
	// Bit 2 4 (26)-Unused.
	// Bit 3 8 (27)-If set, the initialization is performed just before a disk boot is performed. This may change in the ROMmed TOS.
	// Bit 4 10 (28)-Unused.
	// Bit 5 20 (29)-If set, the application is considered to be a desk accessory.
	// Bit 6 40 (30)-If set, the ST will treat the application as a TOS program, and it is assumed that no AES calls will be performed.
	// Bit 7 80 (31)-If set, the ST will treat the application as a "TOS-takes parameters" application. 

	// cartridge start
	dc.l $abcdef42			; 0
	//  dc.b $fa52235f
	dc.l 0					; 4 next
	dc.l start+$8000000	; 8 start code right before the content of the auto folder is executed
	dc.l prg				; c
	dc.w 0					; 10
	dc.w 0					; 12
	dc.l 0					; 14
	dc.b "dummy",0			; 18
	even
//****************************************************************************
	dc.w	firmwareVersion
	jmp		start
	jmp		loadbyBugaboo
	jmp		UDDIReadSector
//****************************************************************************
// do not remove this is a test pattern for build up
	dc.b	$1,$1
	dc.b	$2,$2
	dc.b	$4,$4
	dc.b	$4,$4
	dc.b	$10,$10
	dc.b	$20,$20
	dc.b	$40,$40
	dc.b	$80,$80

start:
	move.w	onoff,d0
	bne		normalstart

	move.l 	#$3d0000,$436		;top of ram
	bsr		initDrive
	rts

	move.l	$44e,a0
	moveq	#1,d0
	moveq	#8,d1
	bsr		UDDIReadSector

	move.l	$44e,a0
	lea		8192*1(a0),a0
	moveq	#1+8*1,d0
	moveq	#8,d1
	bsr		UDDIReadSector

	move.l	$44e,a0
	lea		8192*2(a0),a0
	moveq	#1+8*2,d0
	moveq	#8,d1
	bsr		UDDIReadSector

	move.l	$44e,a0
	lea		8192*3(a0),a0
	moveq	#1+8*3,d0
	moveq	#8,d1
	bsr		UDDIReadSector

loopy:
	move	#$020,$ffff8240.w
	bra		loopy

normalstart:
	movem.l	d0-a6,-(sp)
	move	#0,bugabooLoadFlag
	pea		prg(pc)
	move.w	#$0026,-(sp)
	trap	#$0e
	adda.l	#6,sp
	movem.l	(sp)+,d0-a6

	clr.w	-(sp)
	trap	#1

	rts
loadbyBugaboo:
	move	#1,bugabooLoadFlag
	jmp		loop-startCartridge+$3f0000
//****************************************************************************
prg:
	;copy the cartridge to 3f0000
	;if we run the code at cartridge rom position
	;the atari crashes if the firmware changes

	lea		startCartridge(pc),a0
	lea		$3f0000,a1
	move	#((16384-(startCartridge-$fa0000))/16)-1,d0
.copyLoop:
	move.l	(a0)+,(a1)+
	move.l	(a0)+,(a1)+
	move.l	(a0)+,(a1)+
	move.l	(a0)+,(a1)+
	dbf		d0,.copyLoop
	jmp		$3f0000

startCartridge:
	// check if the zeropage magic is there
	// this is saves during the first run
	// the zero page and restores always
	// the zero page because in case of unclean code run

	cmp.l	#zeroPageMagic,zeroPageMagicPos
	bne		.noZeroPageMagic
	lea		$8,a1
	lea		zeroPageBackup,a0

	move	#(85)-1,d0
.copyzp1:
	movem.l	(a0)+,d1-d7/a2-a6		;7 5
	movem.l	d1-d7/a2-a6,(a1)
	lea		48(a1),a1
	dbf		d0,.copyzp1
	bra		.zeroPageRestored

.noZeroPageMagic
	lea		$8,a0
	lea		zeroPageBackup,a1

	move	#(85)-1,d0
.copyzp2:
	movem.l	(a0)+,d1-d7/a2-a6		;7 5
	movem.l	d1-d7/a2-a6,(a1)
	lea		48(a1),a1
	dbf		d0,.copyzp2

	move.l	#zeroPageMagic,zeroPageMagicPos
.zeroPageRestored:

	// setup screen
	move.w	#$0000,-(sp)
	move.l	$44e,-(sp)
	move.l	$44e,-(sp)
	move.w	#5,-(sp)
	trap	#14
	adda.l	#12,sp

	move	#$2700,sr

	move	#0,$ffff8240.w
	move	#$fff,$ffff8242.w

	// Setup terminal
;	bsr		UDFontReset
	moveq	#0,d0
	bsr		UDVidSetVideoMode
	moveq	#0,d0
	bsr		UDVidSetBGColor
	move	#' ',d0
	bsr		UDDbgScrClear
	move	#0,d0
	move	#0,d1
	move	#7,d2
	lea		vgaStartText(pc),a0
	bsr		UDDbgScrPrint

	// write version numbers
	lea		startTextFpgaVersion(pc),a0
	move.b	UDFPGAVersion,d0
	bsr		writeHexByte

	lea		startTextFpgaVersion+3(pc),a0
	move.b	UDFPGAVersion+1,d0
	bsr		writeHexByte

	lea		startText68kVersion(pc),a0
	move	#firmwareVersion>>8,d0
	bsr		writeHexByte

	lea		startText68kVersion+3(pc),a0
	move	#firmwareVersion,d0
	bsr		writeHexByte

	lea		startText(pc),a0
	move	#0,d0
	move	#0,d1
	move	#1,d2
	bsr		printText
	move	#stateIdle,stateCopy

	move.l	$432,d0
	lea		loadingAdrNumber(pc),a0
	jsr		writeHexLong
	lea		loadingAdrText(pc),a0
	move	#0,d0
	move	#8,d1
	bsr		printText

	bsr		printUploadCounter
	bsr		printStates

	move	readyFlag,d0	;tell the fpga the reset is done

loop:
	; 68k state machine
	move	state,d0
	cmp		#stateIdle,d0
	bne		.noIdle
	move	d0,stateCopy
	move	#$0,$ffff8240.w
.noIdle:
	cmp		#stateStartPrgSetup,d0
	beq		startPrgSetup
	cmp		#stateCopyBlock1,d0
	beq		copyBlock1
	cmp		#stateCopyBlock2,d0
	beq		copyBlock2
	cmp		#stateRunPrg,d0
	beq		runPrg

	bra		loop

//****************************************************************************
startPrgSetup:
	cmp		stateCopy,d0	; check if the last block was the same state? if so
	beq		sendReady		; the fpga is not ready with receiving
	move	d0,stateCopy

	// calculate prg length and allocate memory
	move	#$100,$ffff8240.w
	;alloc symbol table to bugaboo
	clr.l		symboltable

	lea     block1Pos,a0
	tst.l	14(a0)
	beq		.noSymbols
	;alloc memory for symbols
	move.l 	14(a0),d0
	lsl.l	#2,d0			;because of bugaaboo need *4 len
	move.l	d0,-(sp)
	move.w  #$48,-(sp)
	trap    #1
	addq.l  #6,sp
	tst.l 	d0
	bne		.memOk2
	lea		outOfMemText(pc),a0
	bra		doError
.memOk2:
	move.l	d0,symboltable

.noSymbols:

	lea     block1Pos,a0
	lea		64(a0),a0		;because of bugaboo commands
	move.l  2(a0),d0        ;lenght text
	add.l   6(a0),d0        ;lenght data
	add.l   10(a0),d0       ;lenght bss
	add.l   14(a0),d0       ;lenght symboltable
	add.l   #$120,d0         ;add header & code page
	and.l   #$fffffffe,d0   ;lenght for plugin
	move.l	d0,prgLen
	add.l	#$30000,D0		
	move.l  d0,-(sp)		
	move.w  #$48,-(sp)
	trap    #1
	addq.l  #6,sp

	tst.l 	d0
	bne		.memOk
	lea		outOfMemText(pc),a0
	bra		doError
.memOk:
	add.l	#$100,d0
	move.l	d0,prgAdrCopy

	sub.l	#hdrStartText-2,d0
	sub.l	#64,d0				;bugaboo commands
	move.l	d0,bugaBooCmdListPos
	move.l  d0,prgAdr

	bra		copyBlock1Direct

//****************************************************************************
copyBlock1
	cmp		stateCopy,d0	; check if the last block was the same state? if so
	beq		sendReady		; the fpga is not ready with receiving
	move	d0,stateCopy
copyBlock1Direct:
	move	#$010,$ffff8240.w
	lea		block1Pos,a1
	bra		copyBlock

//****************************************************************************
copyBlock2
	cmp		stateCopy,d0	; check if the last block was the same? if so
	beq		sendReady		; the fpga is not ready with receiving
	move	#$001,$ffff8240.w
	move	d0,stateCopy
	lea		block2Pos,a1
	bra		copyBlock

//****************************************************************************
copyBlock
	; copies 8192 bytes to the destination
	move.l	prgAdr,a0
	move	#(170)-1,d0
.copyBlockloop:
	movem.l	(a1)+,d1-d7/a2-a6		;7 5
	movem.l	d1-d7/a2-a6,(a0)
	lea		48(a0),a0
	dbf		d0,.copyBlockloop

	movem.l	(a1)+,d1-d7/a2		;7 5
	movem.l	d1-d7/a2,(a0)
	lea		32(a0),a0
	move.l	a0,prgAdr

	bsr		printUploadCounter
	bsr		printStates
	bra		loop

//****************************************************************************
sendReady:
	move	readyFlag,d0	;send ready flag to fpga
	move	#$111,$ffff8240.w
	bra		loop

//****************************************************************************
runPrg:
	tst		bugabooLoadFlag
	bne		.no12
	move.l	#$3e8000,a7
.no12:
	bsr		printStates

	move	readyFlag,d0	;send ready flag to fpga
	move	readyFlag,d0	;send ready flag to fpga
	move	readyFlag,d0	;send ready flag to fpga
	move	readyFlag,d0	;send ready flag to fpga

	move.l 	#$3e0000,$436		;top of ram
	move	#$345,$ffff8240.w	
	// build up code page
	move.l	prgAdrCopy,a0
	sub.l	#$100,a0

	;a0 = code page
	;a1 = file header
	;a2 = pointer to the segments (moving)
	move.l	a0,p_lowtpa(a0)

	move.l	a0,a1
	add.l 	prgLen,a1
	move.l	a1,p_hitpa(a0)

	;text segment
	move.l	prgAdrCopy,a2
	move.l 	a2,p_tbase(a0)

	move.l	prgAdrCopy,a1
	sub.l	#hdrStartText-2,a1
	move.l 	hdrSizeText(a1),p_tlen(a0)
	add.l	hdrSizeText(a1),a2

	;data segment
	move.l 	a2,p_dbase(a0)
	move.l 	hdrSizeData(a1),p_dlen(a0)
	add.l	hdrSizeData(a1),a2
	
	;bss seqment
	move.l	a2,p_bbase(a0)
	move.l 	hdrSizeBss(a1),p_blen(a0)

	lea		emptyEnvStr(pc),a3
	move.l	a3,p_env(a0)
	clr.l	p_cmdlin(a0)

	movea.l $04F2.w,A2      ;get Sysbase
	movea.l 8(A2),A2        ;get rom start
	move.l	$28(a2),a2
	tst		bugabooLoadFlag
	bne		.no1
	move.l	a0,(a2)			;set act pd 
.no1:
	tst		bugabooLoadFlag
	bne		.no11
	move	#$0,sr
	move.l	#$3f0000,a7
.no11:

	// when start with bugaboo copy the symbols to the allocated memory
	tst		bugabooLoadFlag
	beq		.no4
	tst.l	symboltable
	beq		.no4
	movem.l d0-a6,-(a7)
	
	move.l	prgAdrCopy,a0
	sub.l	#hdrStartText-2,a0
	lea		(a0),a2
	move.l	symboltable,a1
	move.l 	2(A0),D0        ;add text seg
    add.l   6(A0),D0        ;add data seg
	add.l	d0,a0			;sym pos
	move.l	14(a2),d0		;len
	move.l	d0,symboltablelen
	add.l	#hdrStartText-2,a0
	add.l 	#10,d0
	move.l	a0,a6
.copysym
	move.b	(a0)+,(a1)+
	sub.l	#1,d0
	bne		.copysym
	movem.l	(a7)+,d0-a6
.no4:
	// relocate programm
	move.l	prgAdrCopy,a0
	sub.l	#hdrStartText-2,a0
	bsr		relocate

	move	#stateIdle,stateCopy
	move.l	prgAdrCopy,a0
	sub.l	#$100,a0

	;clear bss
	move.l	p_bbase(a0),a1
	move.l	p_blen(a0),d0
	lsr.l	#4,d0
	tst.l	d0
	beq.s	.nobbs
	add.l	#4,d0
.clrbss:
	clr.l	(a1)+
	clr.l	(a1)+
	clr.l	(a1)+
	clr.l	(a1)+
	subq.l	#1,d0
	bne.s	.clrbss
.nobbs:
	move.l	a0, myBasePage

	tst		bugabooLoadFlag
	bne.s	.no2
	move.l	a0,-(a7)	;put code page to stack
	clr.l	-(a7)		;clear something ;)
.no2:
	moveq	#0,d0
	moveq	#0,d1
	moveq	#0,d2
	moveq	#0,d3
	moveq	#0,d4
	moveq	#0,d5
	moveq	#0,d6
	moveq	#0,d7
	move.l	d0,a0
	move.l	d0,a1
	move.l	d0,a2
	move.l	d0,a3
	move.l	d0,a4
	move.l	d0,a5
	move.l	d0,a6
	move.l	prgAdrCopy,a6
	tst		bugabooLoadFlag
	bne		.no3
	jmp		(a6)
.no3:
	rts

//****************************************************************************
doError:
	move	#0,d0
	move	#10,d1
	bsr		printText
.errorLoop:
	bra		.errorLoop

//****************************************************************************
hex: dc.b "0123456789ABCDEF"
vgaStartText:
	dc.b "ultraDev VGA Debug Screen",10
	dc.b "-------------------------",0

startText:	dc.b "ultraDev",10
	dc.b         "--------",10
	dc.b "FPGA Version "
startTextFpgaVersion:
	dc.b "00.00",10
	dc.b "68k  Version "
startText68kVersion:
	dc.b "00.00",10,10
	dc.b "Waiting...",10,10,10,10,10,10,10,10,10,10,10,10,10,10
	dc.b "(c) 2020 cream",0
	even
state68KText: dc.b "68k state: ",0

	;      0123456789abcdef
state68KTexts:	dc.b	"Idle           ",0
	dc.b	"StartPrgSetup  ",0
	dc.b	"CopyBlock1     ",0
	dc.b	"CopyBlock2     ",0
	dc.b	"RunPrg         ",0
	even

fpgaStateText: dc.b	"FPGA state:",10
fpgaStateNumber: dc.b	"00 00",0
lenText: dc.b	"0000 0000",0

loadingAdrText: dc.b	"Loading address:",10
loadingAdrNumber: dc.b	"00000000",0

	even

outOfMemText: dc.b "Out of memory!...",0
	even

emptyEnvStr: dc.b	"PATH=",0
	dc.b	"A:\",0,0
	even

//****************************************************************************
	include "screen.s"
;	include "disassemble.s"
;	include "font.s"
	include "relocate.s"
	include "drive.s"

	include "../68klib/UDDefines.s"
	include "../68klib/UDVideo.s"
	include "../68klib/UDDebugScreen.s"
	include "../68klib/UDFont.s"
	include "../68klib/UDDriveImage.s"
	


//****************************************************************************
